@@ -1,5 +1,5 @@
 use std::{
-    io::{self, Write},
+    io::{self, Read, Write},
     path::PathBuf,
     thread::sleep,
     time::Duration,
@@ -33,12 +33,38 @@ fn main() {
         }
         "stream chunk to stderr then wait for file" => {
             eprintln!("foo");
-            io::stdout().flush().unwrap();
             let file = PathBuf::from("./file");
             while !file.exists() {
                 sleep(Duration::from_secs_f32(0.1));
             }
         }
+        "reverse" => {
+            let mut input = Vec::new();
+            io::stdin().read_to_end(&mut input).unwrap();
+            input.reverse();
+            io::stdout().write_all(&input).unwrap();
+            io::stdout().flush().unwrap();
+        }
+        "wait until stdin is closed" => {
+            while !stdin_is_closed() {}
+            println!("stdin is closed");
+        }
         arg => panic!("cradle_test_helper: invalid arg: {}", arg),
     }
 }
+
+fn stdin_is_closed() -> bool {
+    #[cfg(unix)]
+    {
+        use nix::poll::{poll, PollFd, PollFlags};
+        let mut poll_fds = [PollFd::new(0, PollFlags::all())];
+        poll(&mut poll_fds, 0).unwrap();
+        if let Some(events) = poll_fds[0].revents() {
+            events.contains(PollFlags::POLLHUP)
+        } else {
+            false
+        }
+    }
+    #[cfg(windows)]
+    panic!("stdin_is_closed is not supported on windows")
+}
